/**********************************************************************************************/
/* FBL_ToFromString.h                                                      				 	  */
/*                                                                       					  */
/* Foundation Base Library  ( FBL )		                                  					  */
/*                                                                       					  */
/* Copyright Paradigma, 1993-2002                                       					  */
/* All Rights Reserved.                                                 					  */
/**********************************************************************************************/

#ifndef __FBL_ToFromString_h 
	#define __FBL_ToFromString_h

#if VSDK_FRAMEWORK_STYLE
#	include <VSDK/FBL.h>
#else
#	include <FBL.h>
#endif // VSDK_FRAMEWORK_STYLE

#include <stdio.h>

#if defined(__MWERKS__)
	#pragma options align=power
#endif


/**********************************************************************************************/
// This structure specify how Date and Time fields will be converted TO/FROM strings.
// This conversion affects any string operation of kernel, including Import/Export and SQL queries.
 
struct FBL_DTFormat
{
		uchar				mDateFormat;
		char				mDateSep;		// deafult is '/'
		char				mTimeSep;		// default is ':'
	
	public:///////////////////////////////////////////////////////////////////////////////////// 
							
		ushort				GetBound( void ) const { return mBound; }	
		void				SetBound( ushort inBound = 20 ) { mBound = inBound; }
				
		void 				GetCentury( ulong& outCentury1, ulong& outCentury2, ushort& outBound ) const
							{
								outCentury1 = mCentury1;
								outCentury2	= mCentury2;
								outBound 	= mBound;
							}	
							
							
							
		void				SetCentury( ulong inCentury1 = 1900, ulong inCentury2 = 2000, ushort inBound = 20) 
							{
								CHECK_FBL( inCentury2 - inCentury1 == 100 ); 

								mCentury1 = inCentury1;
								mCentury2 = inCentury2;
								mBound 	  = inBound;
							}
	
							/** Peroforms autocorrection of Year from yy to YYYYY format of full year.
								This is comfortable for user. 
								Auto correction works only for year in range 0..99.
								Note, that for negative year there is NO auto correction. 
								You can turn OFF auto correction at all if set mBound to ZERO. */
		void				AutoCentury( long& ioYear )
							{		
								
								if( mBound && 0 <= ioYear && ioYear <= 99 )
								{
									if( ioYear <= mBound )
										ioYear += (long) mCentury2;
									else if( ioYear > mBound )
										ioYear += (long) mCentury1;	
								}
							}
							
	
	private:////////////////////////////////////////////////////////////////////////////////////	 
	
		// The next 3 parameters allow us to do auto-adding of century: YY -> YYYY
		// If these parameters are zero then auto expanding is not performed.
		// This can be used if for historical database.
		
		ulong				mCentury1;			// default 1900
		ulong				mCentury2;			// default 2000

		ushort				mBound;				// default 20			00 => 2000, 20 => 2020
												//						21 => 1921, 99 => 1999
												
};


/**********************************************************************************************/
// This keeps settings default for current system.
// Kernel get them on initialisation.
FBL_EXP extern FBL_DTFormat* 	gSystemDTFormat;

// Each database has property mDTFormat, 
// global pointer gDTFormat switched on current database property
// before FBL_SetString() or FBL_GetString() are called for Date or Time fields.
FBL_EXP extern FBL_DTFormat* 	gDTFormat;


/**********************************************************************************************/
// This methods get only pointer to memory buffer of BaseField.
// They don't get pointer to the BaseField itself, thanks to this this methods can be called
// as from kernel methods which use clases FBL_BaseField so and from SDK methods
// which use classes VDK_Field.

// !!! ATTENTION: !!!
// You must not call these methods from your code.
// Use instead VDK_Field::SetString() and VDK_Field::GetString(), which before call of
// this bottleneck methods will prepare gDTFormat pointer for Date and Time fields.

// NOTE: sources of this bottleneck methods are opened, so you can change them if you need,
// and your changes will works everywhere including import/export.

FBL_EXP 
void FBL_SetString( 
		void*		inFieldDataPtr, 
		uchar		inFieldType, 
		ushort 		inBytes,
		const char* inString );


FBL_EXP 
char* FBL_GetString( 
		void*		inFieldDataPtr, 
		uchar		inFieldType, 
		char*		outString,
		ulong 		inBufferBytes = (ulong) -1 );


/**********************************************************************************************/
inline bool FBL_StrToBool( const char* p )
{
	bool res = false;

	if( *p == '1' )
	{ 
		res = true; 
		
	}else if( 	(p[0] == 'T' || p[0] == 't') &&	
				(p[1] == 'R' || p[1] == 'r') &&
				(p[2] == 'U' || p[2] == 'u') &&
				(p[3] == 'E' || p[3] == 'e') ) 
	{
		res = true;
	} 
		
	return res;
		
} 


/**********************************************************************************************/
void InitDateStaticData( void );		// for internal use.
void DestroyDateStaticData( void );		// for internal use.


/**********************************************************************************************/
inline void FBL_StringToDate( const char* inString, long* Year, ushort* Month, ushort* Day )
{
	char c1 = 0, c2 = 0;
	bool neg = false;

	if( *inString == '-' ){
		inString++;
		neg = true;
	} 
	
	switch( gDTFormat->mDateFormat )
	{
		case 0:	sscanf( inString, "%hu%c%hu%c%ld", Month, &c1, Day,   &c2, Year );
				break;
		case 1:	sscanf( inString, "%hu%c%hu%c%ld", Day,   &c1, Month, &c2, Year );
				break;
		case 2:	sscanf( inString, "%ld%c%hu%c%hu", Year,  &c1, Month, &c2, Day );
				break;
	}
	
	if( neg ) 
		*Year = -(*Year);  

	gDTFormat->AutoCentury( *Year );
}


/**********************************************************************************************/
inline void FBL_StringToTime( const char* inString, ushort* H, ushort* M, ushort* S )
{
	char c1 = 0, c2 = 0;

	sscanf( (char*)inString, "%hu%c%hu%c%hu", H, &c1, M, &c2, S );
}


/**********************************************************************************************/
inline ulong FBL_DateToString( char* outString, long Year, ushort Month, ushort Day )
{
	ulong Count = 0;
	
	char c = gDTFormat->mDateSep;

	switch( gDTFormat->mDateFormat )
	{
		case 0:	Count = (ulong) sprintf( outString, "%.2u%c%.2u%c%.4li", Month, c, Day, c, Year );
				break;
		case 1:	Count = (ulong) sprintf( outString, "%.2u%c%.2u%c%.4li", Day, c, Month, c, Year );
				break;
		case 2:	Count = (ulong) sprintf( outString, "%.4li%c%.2u%c%.2u", Year, c, Month, c, Day );
				break;
	}

	return Count;
}

/**********************************************************************************************/
inline ulong FBL_TimetoString( char* outString, ushort H, ushort M, ushort S )
{
	char c = gDTFormat->mTimeSep;

	return (ulong) sprintf( (char*)outString, "%.2u%c%.2u%c%.2u", H, c, M, c, S );
}


/**********************************************************************************************/
#if defined(__MWERKS__)
	#pragma options align=reset
#endif

#endif	// __FBL_ToFromString_h
